// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.MessageDialogWithToggle;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.AnnotationModel;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.texteditor.ITextEditorActionConstants;

import eu.etaxonomy.taxeditor.annotatedlineeditor.AnnotatedLineDocumentProvider;
import eu.etaxonomy.taxeditor.annotatedlineeditor.AnnotatedLineEditor;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotation;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotationModel;
import eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.TaxonEditorInput;
import eu.etaxonomy.taxeditor.model.IDerivedUnitFacadePart;
import eu.etaxonomy.taxeditor.model.IDirtyMarkable;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasFactualData;
import eu.etaxonomy.taxeditor.model.IPartContentHasMedia;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author p.ciardelli
 * @created 07.07.2009
 * @version 1.0
 */
public class BulkEditor extends AnnotatedLineEditor implements IPartContentHasDetails,
        IDirtyMarkable, IDerivedUnitFacadePart, IPartContentHasFactualData, IPartContentHasMedia {

	/** Constant <code>ID="bulkeditor.editor"</code> */
	public static final String ID = "bulkeditor.editor";

	private boolean isInitialFocus = true;

	private BulkEditorSearch searchBar = null;

	private IPropertyChangeListener markerPreferenceListener;

	private boolean isDirty;

	public BulkEditor() {
		super(CdmStore.createConversation());
	}

	/** {@inheritDoc} */
	@Override
	protected void initializeEditor() {
		super.initializeEditor();

		/**
		 * see AbstractTextEditor javadoc for explanation of context menu ids
		 */
		setEditorContextMenuId("#BulkEditorContext");

//		setEntityCreatorService(new BulkEditorEntityCreatorService());

		setLineDisplayStrategy(new BulkEditorLineDisplay(this));

		setSourceViewerConfiguration(new BulkEditorViewerConfiguration(lineDisplayStrategy));
	}

	/** {@inheritDoc} */
	@Override
    protected ISourceViewer createSourceViewer(Composite parent,
			IVerticalRuler ruler, int styles) {
		ISourceViewer viewer = super.createSourceViewer(parent, ruler, styles);
		if (getEditorInput().isMergingEnabled()) {
			addToggleMergeCandidateListener(ruler.getControl());
		}
		return viewer;
	}

	/** {@inheritDoc} */
	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {

		if (!(input instanceof AbstractBulkEditorInput)) {
			throw new PartInitException("Invalid Input: Must be BulkEditorInput");
		}

		super.init(site, input);
	}

	/** {@inheritDoc} */
	@Override
	public void createPartControl(Composite parent) {

		parent.setLayout(new GridLayout());

		Composite layoutComposite = new Composite(parent, SWT.NONE);
		layoutComposite.setLayout(new GridLayout());

		GridData gridData = new GridData(SWT.FILL, SWT.TOP, true, false);
		layoutComposite.setLayoutData(gridData);

//		layoutComposite.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_BLUE));


		searchBar = new BulkEditorSearch(this, layoutComposite, SWT.NONE);
//		layoutComposite.setBackground(parent.getDisplay().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));

		super.createPartControl(parent);

		// Set viewer composite to fill grid. Unfortunately it is private and we have to do a little hack here.
		for (Control control : parent.getChildren()) {
			if (control instanceof Composite &&
						!(control.equals(layoutComposite))) {
				control.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		if (markerPreferenceListener != null ) {
			PreferencesUtil.getPreferenceStore().removePropertyChangeListener(markerPreferenceListener);
		}
		super.dispose();
	}


	/** {@inheritDoc} */
	@Override
	public boolean isEditable() {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public void setFocus() {
	    conversation.bind();
		searchBar.setFocus();

		// TODO find a better place to put this - this dialog should be shown after initial contents of
		//	Editor are displayed
		if (isInitialFocus) {
			displayWarningDialog();
			isInitialFocus = false;
		}
	}

	private void displayWarningDialog() {
		IPreferenceStore prefs = PreferencesUtil.getPreferenceStore();
		if (!prefs.getBoolean(PreferencesUtil.HIDE_BULKEDITOR_INFO)) {
			String msg = "The Bulk Editor allows you to edit objects used to reference other objects, such as names, references, and authors.\n\n" +
							"Any changes you make to an object in the Bulk Editor will be displayed wherever the object is used.\n\n" +
							"For instance, a reference may be displayed with both a name and a descriptive element. If the reference name is changed here, the display of both the name and the descriptive element will be affected.";
			MessageDialogWithToggle dialog = MessageDialogWithToggle.openOkCancelConfirm
												(getSite().getShell(), "Bulk Editor", msg, "Do not show this message again",
														false, null, PreferencesUtil.HIDE_BULKEDITOR_INFO);
			if (dialog.getReturnCode() == Window.OK) {
				prefs.setValue(PreferencesUtil.HIDE_BULKEDITOR_INFO, dialog.getToggleState());
			}
		}
	}

	private void addToggleMergeCandidateListener(Control control) {
		control.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseDoubleClick(MouseEvent e) {
				StyledText textWidget = getSourceViewer().getTextWidget();
				int line = textWidget.getLineIndex(e.y);
				toggleMergeCandidateAnnotation(line);
			}
		});
	}

	public void toggleMergeCandidateAnnotation(int line) {

		IDocument document = getSourceViewer().getDocument();
		LineAnnotationModel model =
				(LineAnnotationModel) getSourceViewer().getAnnotationModel();

		if(model != null){
			Annotation annotation = model.getAnnotationAtLine(line, document);

			if (annotation != null) {
				if (annotation.getType().equals(IBulkEditorConstants.TYPE_MERGE_CANDIDATE)) {
					model.changeAnnotationType(
							annotation, LineAnnotation.TYPE_GENERIC);
				} else {
					model.changeAnnotationType(
							annotation, IBulkEditorConstants.TYPE_MERGE_CANDIDATE);
				}
			}
		}
	}

	@Override
	public boolean isDirty() {
	    if(isDirty){
	        return isDirty;
	    }
	    else{
	        return super.isDirty();
	    }
	}

	@Override
    public void forceDirty(){
	    isDirty = true;
	    firePropertyChange(PROP_DIRTY);
	}

	@Override
	public void doSave(IProgressMonitor progressMonitor) {
	    isDirty = false;

	    super.doSave(progressMonitor);

	    selectFirstItem();

		getSourceViewer().getTextWidget().setFocus();

	}

	/** {@inheritDoc} */
	@Override
    public void changed(Object object) {
//		this.dirty = dirty;
		AnnotatedLineDocumentProvider p = (AnnotatedLineDocumentProvider) getDocumentProvider();
		p.changed(object);
//		firePropertyChange(PROP_DIRTY);
	}

	/** {@inheritDoc} */
	public void performSearch(BulkEditorQuery query) {
		if (query != null) {

			// TODO check if dirty, prompt save
			if (isDirty()) {
				boolean proceed = MessageDialog.openQuestion(getEditorSite().getShell(),
						"Save changes", "You have made changes that must be saved before this query can be executed. Would you like to proceed?");
				if (proceed) {
					doSave(null);
				} else {
					return;
				}
			}
			conversation.commit();
			conversation.clear();

			getEditorInput().performSearch(query);

			refresh();

			selectFirstItem();

			getSourceViewer().getTextWidget().setFocus();
		}
	}

	private void selectFirstItem() {
		ITextSelection selection = new TextSelection(0, 0);
		getSelectionProvider().setSelection(selection);
	}

	public void refresh() {
		if(getDocumentProvider().getAnnotationModel(getEditorInput()) != null){
			((AnnotationModel) getDocumentProvider().getAnnotationModel(getEditorInput())).removeAllAnnotations();
		}

		setInput(getEditorInput());
	}

	/** {@inheritDoc} */
	@Override
	public AbstractBulkEditorInput getEditorInput() {
		return (AbstractBulkEditorInput) super.getEditorInput();
	}

	@Override
    protected void editorContextMenuAboutToShow(IMenuManager menu) {
		super.editorContextMenuAboutToShow(menu);
		menu.remove(ITextEditorActionConstants.SHIFT_RIGHT);
		menu.remove(ITextEditorActionConstants.SHIFT_LEFT);
		menu.remove(ITextEditorActionConstants.CONTEXT_PREFERENCES);
	}

	@Override
	public boolean canAttachMedia() {
	    return getEditorInput() instanceof TaxonEditorInput?true:false;
	}
}
