package eu.etaxonomy.taxeditor.bulkeditor.input;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import org.eclipse.jface.action.IContributionItem;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.menus.CommandContributionItem;
import org.eclipse.ui.menus.CommandContributionItemParameter;

import eu.etaxonomy.cdm.api.service.IAgentService;
import eu.etaxonomy.cdm.api.service.IGroupService;
import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.IUserService;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.common.Group;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorConstants;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

public enum BulkEditorInputType {
	AGENT(Messages.BulkEditorInputType_0, AgentEditorInput.ID), 
	REFERENCE(Messages.BulkEditorInputType_1, ReferenceEditorInput.ID), 
	NAME(Messages.BulkEditorInputType_2, NameEditorInput.ID), 
	NAME_RELATIONSHIP(Messages.BulkEditorInputType_3, NameRelationshipEditorInput.ID), 
	OCCURRENCE(Messages.BulkEditorInputType_4, OccurrenceEditorInput.ID), 
	USER(Messages.BulkEditorInputType_5, UserEditorInput.ID), 
	GROUP(Messages.BulkEditorInputType_6, GroupEditorInput.ID),
	TAXON(Messages.BulkEditorInputType_7, TaxonEditorInput.ID);

	public String id;
	public String label;
	public ResourceBundle resourceBundle; 

	BulkEditorInputType(String label, String id) {
		this.id = id;
		this.label = label;
	}

	public IContributionItem createContributionItem(){ 
		return createContributionItem(label, id);
	}
	
	/**
	 * @param key
	 * @param object
	 * @return
	 */
	private IContributionItem createContributionItem(String label,
			String inputType) {
		CommandContributionItemParameter parameter = new CommandContributionItemParameter(
				PlatformUI.getWorkbench().getActiveWorkbenchWindow(), null,
				IBulkEditorConstants.DYNAMIC_OPEN_MENU_ID, SWT.NONE);

		parameter.label = label;

		Map parameters = new HashMap();
		parameters.put(IBulkEditorConstants.INPUT_TYPE_PARAMETER_ID, inputType);
		parameter.parameters = parameters;

		return new CommandContributionItem(parameter);
	}
	
	public static BulkEditorInputType getById(String id) {

		for (BulkEditorInputType type : values()) {
			if (id.equals(type.id)) {
				return type;
			}
		}

		return null;
	}

	public static BulkEditorInputType getByType(Class clazz) {
		if (Reference.class.isAssignableFrom(clazz)) {
			return REFERENCE;
		} else if (TaxonNameBase.class.isAssignableFrom(clazz)) {
			return NAME;
		} else if (AgentBase.class.isAssignableFrom(clazz)) {
			return AGENT;
		} else if (SpecimenOrObservationBase.class.isAssignableFrom(clazz)) {
			return OCCURRENCE;
		} else if (NameRelationship.class.isAssignableFrom(clazz)) {
			return NAME_RELATIONSHIP;
		} else if (Group.class.isAssignableFrom(clazz)) {
			return GROUP;
		} else if (User.class.isAssignableFrom(clazz)) {
			return USER;
		} else if (Taxon.class.isAssignableFrom(clazz)){
			return TAXON;
		}
		return null;
	}

	public static BulkEditorInputType getByInput(IEditorInput input) {
		if (input instanceof ReferenceEditorInput) {
			return REFERENCE;
		} else if (input instanceof NameEditorInput) {
			return NAME;
		} else if (input instanceof AgentEditorInput) {
			return AGENT;
		} else if (input instanceof OccurrenceEditorInput) {
			return OCCURRENCE;
		} else if (input instanceof NameRelationshipEditorInput) {
			return NAME_RELATIONSHIP;
		} else if (input instanceof UserEditorInput) {
			return USER;
		} else if (input instanceof GroupEditorInput) {
			return GROUP;
		} else if (input instanceof TaxonEditorInput){
			return TAXON;
		}
		return null;
	}

	public static AbstractBulkEditorInput getInput(
			BulkEditorInputType inputType) {
		switch (inputType) {
		case REFERENCE:
			return new ReferenceEditorInput();
		case NAME:
			return new NameEditorInput();
		case AGENT:
			return new AgentEditorInput();
		case OCCURRENCE:
			return new OccurrenceEditorInput();
		case NAME_RELATIONSHIP:
			return new NameRelationshipEditorInput();
		case USER:
			return new UserEditorInput();
		case GROUP:
			return new GroupEditorInput();
		case TAXON:
			return new TaxonEditorInput();
		default:
			throw new IllegalStateException(
					"No input class for the given input type defined."); //$NON-NLS-1$
		}
	}
	
	public static Class getServiceClass(
			BulkEditorInputType inputType) {
		switch (inputType) {
		case REFERENCE:
			return IReferenceService.class;
		case NAME:
			return INameService.class;
		case AGENT:
			return IAgentService.class;
		case OCCURRENCE:
			return IOccurrenceService.class;
		case NAME_RELATIONSHIP:
			return INameService.class;
		case USER:
			return IUserService.class;
		case GROUP:
			return IGroupService.class;
		case TAXON:
			return ITaxonService.class;
		default:
			throw new IllegalStateException(
					"No input class for the given input type defined."); //$NON-NLS-1$
		}
	}
}